#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 7/15/22

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import os, sys,time
import random
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_rusty_text_effect(customtext, font, textcolor):
    
    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    #Create image
    img = pdb.gimp_image_new(1, 1, RGB_IMAGE)
    
    #Create the text
    size = 200
    pdb.gimp_context_set_foreground(textcolor)
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 100, 
                                        TRUE, size, PIXELS, font)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 20)
   
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)

    gimp.Display(img)
   
    #Create the background
    wh = img.width / 2
    hh = img.height / 2
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, backLayer, None, 1)
    pdb.gimp_context_set_foreground("#919191")
    pdb.gimp_context_set_background(textcolor)
    pdb.gimp_edit_blend(backLayer, 
								0,  #blend-mode - blend_fg_bg_rgb
	                            0,  #paint-mode - layer_mode_normal
	                            2,  #gradient type - radial
	                          100,  #opacity
							    0,  #offset
								0,  #repeat
							FALSE,  #reverse
							FALSE,  #supersample
								3,  #max-depth
							 0.20,  #threshold
						     TRUE,  #dither
				               wh,  #width1
				           hh-100,  #height1
					 img.width-50,  #width2
				    img.height-50)  #height2
       
    grunge_Layer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Grunge", 100, LAYER_MODE_DIFFERENCE)
    pdb.gimp_image_insert_layer(img, grunge_Layer, None, 1)
    pdb.gimp_drawable_fill(grunge_Layer, FILL_TRANSPARENT)
    pdb.plug_in_gmic_qt(img, grunge_Layer, 1, 0, "-fx_cracks 50,1,255,255,255,128,0,0,50,50")
    pdb.plug_in_gmic_qt(img, grunge_Layer, 1, 0, "-souphead_droste10 66,100,1,1,1,0,0,2,0,6,1,5,1,0,90,6,0,0,0,1,0,0,1,0,0,0,0,0,1,0,0")
    grunge_Layer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Grunge", 100, LAYER_MODE_SOFTLIGHT)
    pdb.gimp_image_insert_layer(img, grunge_Layer2, None, 1)
    pdb.gimp_context_set_foreground("#df5312")
    pdb.gimp_drawable_fill(grunge_Layer2, FILL_FOREGROUND)
    grunge_Layer = pdb.gimp_image_merge_down(img, grunge_Layer2, CLIP_TO_IMAGE)
    pdb.plug_in_emboss(img, grunge_Layer, 30, 45, 20, FALSE)
    pdb.gimp_drawable_hue_saturation(grunge_Layer, ALL_HUES, 10, 10, 100, 0)
    backLayer = pdb.gimp_image_merge_down(img, grunge_Layer, CLIP_TO_IMAGE)
    
    #Duplicate the text
    textCopy = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, textCopy, None, 0)
    pdb.gimp_layer_set_mode(textCopy, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(textCopy, 100)
    pdb.gimp_image_select_item(img, 2, textCopy)
    pdb.gimp_selection_grow(img, 2)
    pdb.gimp_context_set_foreground("#493728")
    pdb.gimp_edit_bucket_fill(textCopy,  
                        BUCKET_FILL_FG,  #fill_mode
                     LAYER_MODE_NORMAL,  #paint_mode
                                   100,  #opacity
                                   255,  #threshold
                                 FALSE,  #sample_merged
                                     0,  #x
                                     0)  #y
    pdb.gimp_selection_none(img)
    
    #Text drop shadow
    color = "#000000"
    opacity = 75.0
    contour = 0
    noise = 0.0
    mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
    spread = 40.0
    size = 8
    offsetangle = 120.0
    offsetdist = 9.0
    knockout = TRUE
    merge = FALSE
    pdb.python_layerfx_drop_shadow(img, textLayer, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    
    #Text inner glow
    color = ("#6c5940")
    opacity = 75.0
    contour = 0
    noise = 0.0
    mode = 5        #LAYER_MODE_OVERLAY_LEGACY
    source = 0
    choke = 0.0
    size = 8
    merge = TRUE
    pdb.python_layerfx_inner_glow(img, textLayer, color, opacity, contour, noise, mode, source, choke, size, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    
    #Create the rust texture
    new_size = 300
    rust_Layer = pdb.gimp_layer_new(img, new_size, new_size, RGBA_IMAGE, "Rust texture", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, rust_Layer, None, 0)
    pdb.gimp_drawable_fill(rust_Layer, FILL_TRANSPARENT)
    pdb.plug_in_plasma(img, rust_Layer, random.randrange(1,999999999), 5)
    pdb.gimp_drawable_desaturate(rust_Layer,  DESATURATE_LIGHTNESS)
    #Shadow color balance
    pdb.gimp_drawable_color_balance(rust_Layer, 
                                             0,   #transfer_mode - shadows
                                          TRUE,   #preserve_lum
                                           100,   #cyan_red
                                           -15,   #magenta_green,
                                           -67)   #yellow_blue
	#Midtones color balance
    pdb.gimp_drawable_color_balance(rust_Layer, 
                                             1,   #transfer_mode - midtones
                                          TRUE,   #preserve_lum
                                           100,   #cyan_red
                                           -17,   #magenta_green,
                                           -55)   #yellow_blue                                 
	#Hightlights color balance
    pdb.gimp_drawable_color_balance(rust_Layer, 
                                             2,   #transfer_mode - hightlight
                                          TRUE,   #preserve_lum
                                           100,   #cyan_red
                                           -60,   #magenta_green,
                                          -100)   #yellow_blue      
    
    pdb.plug_in_colortoalpha(img, rust_Layer, ("#ffffff"))
    non_empty = pdb.gimp_edit_copy(rust_Layer)
    pdb.gimp_image_remove_layer(img, rust_Layer)
    
    #Text bevel and emboss
    style = 1                      #Inner bevel
    depth = 65
    direction = 0                  #Up
    size = 7
    soften = 1
    angle = 50.0
    altitude = 45.0
    glosscontour = 0                #Linear
    highlightcolor = ("#ffffff")
    highlightmode = 4                #LAYER_MODE_SCREEN_LEGACY
    highlightopacity = 80.0
    shadowcolor = ("#7b7b7b")
    shadowmode = 3                  #LAYER_MODE_MULTIPLY_LEGACY
    shadowopacity = 35.0
    surfacecontour = 0              #Linear
    use_texture = TRUE
    pattern = _("Clipboard Image")
    scale = 50.0
    tex_depth = 50.0
    invert = FALSE
    merge = TRUE
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
                                    style, depth, direction, size, soften, angle, altitude, glosscontour, highlightcolor, highlightmode, 
                                    highlightopacity, shadowcolor, shadowmode, 
                                    shadowopacity, surfacecontour, use_texture, pattern, scale, tex_depth, invert, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")

    #Text pattern overlay
    pattern =  _("Clipboard Image")
    opacity = 100.0
    mode = 5                       #LAYER_MODE_OVERLAY_LEGACY,
    scale = 50.0
    interpolation_type = INTERPOLATION_NONE
    merge = TRUE
    pdb.python_layerfx_pattern_overlay(img, textLayer, pattern, opacity, mode, scale, interpolation_type, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    
    #Text pattern overlay
    pattern =  _("Clipboard Image")
    opacity = 100.0
    mode = 5                       #LAYER_MODE_OVERLAY_LEGACY,
    scale = 30.0
    interpolation_type = INTERPOLATION_NONE
    merge = TRUE
    pdb.python_layerfx_pattern_overlay(img, textCopy, pattern, opacity, mode, scale, interpolation_type, merge)
    textCopy = pdb.gimp_image_get_layer_by_name(img, "Text #1")
    pdb.gimp_layer_set_mode(textCopy, LAYER_MODE_SUBTRACT)
    
    #Make it realistic
    pdb.gimp_drawable_hue_saturation(textCopy, ALL_HUES, 0, -50, 100, 0)
    textCopy = pdb.gimp_image_merge_down(img, textCopy, CLIP_TO_IMAGE)
    
    #Resets the previous user defaults
    pdb.gimp_context_pop()
    
    
def pm_rusty_text_effect_alt(img, layer, color):

    #Start undo
    #pdb.gimp_image_undo_group_start(img)

    #Get the bottom layer
    wh = img.width / 2
    hh = img.height / 2
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_image_set_active_layer(img, backLayer)
    
    #Create a rusty background
    pdb.gimp_context_set_foreground("#919191")
    pdb.gimp_context_set_background(color)
    pdb.gimp_edit_blend(backLayer, 
								0,  #blend-mode - blend_fg_bg_rgb
	                            0,  #paint-mode - layer_mode_normal
	                            2,  #gradient type - radial
	                          100,  #opacity
							    0,  #offset
								0,  #repeat
							FALSE,  #reverse
							FALSE,  #supersample
								3,  #max-depth
							 0.20,  #threshold
						     TRUE,  #dither
				               wh,  #width1
				           hh-100,  #height1
					 img.width-50,  #width2
				    img.height-50)  #height2
       
    grunge_Layer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Grunge", 100, LAYER_MODE_DIFFERENCE)
    pdb.gimp_image_insert_layer(img, grunge_Layer, None, 1)
    pdb.gimp_drawable_fill(grunge_Layer, FILL_TRANSPARENT)
    pdb.plug_in_gmic_qt(img, grunge_Layer, 1, 0, "-fx_cracks 50,1,255,255,255,128,0,0,50,50")
    pdb.plug_in_gmic_qt(img, grunge_Layer, 1, 0, "-souphead_droste10 66,100,1,1,1,0,0,2,0,6,1,5,1,0,90,6,0,0,0,1,0,0,1,0,0,0,0,0,1,0,0")
    grunge_Layer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Grunge", 100, LAYER_MODE_SOFTLIGHT)
    pdb.gimp_image_insert_layer(img, grunge_Layer2, None, 1)
    pdb.gimp_context_set_foreground("#df5312")
    pdb.gimp_drawable_fill(grunge_Layer2, FILL_FOREGROUND)
    grunge_Layer = pdb.gimp_image_merge_down(img, grunge_Layer2, CLIP_TO_IMAGE)
    pdb.plug_in_emboss(img, grunge_Layer, 30, 45, 20, FALSE)
    pdb.gimp_drawable_hue_saturation(grunge_Layer, ALL_HUES, 10, 10, 100, 0)
    backLayer = pdb.gimp_image_merge_down(img, grunge_Layer, CLIP_TO_IMAGE)
    
    #Make active the text 
    position = pdb.gimp_image_get_item_position(img, layer)
    textLayer = layer
    pdb.gimp_image_set_active_layer(img, textLayer)    
    pdb.gimp_item_set_name(textLayer, "Text")
    
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    
    pdb.gimp_layer_resize_to_image_size(textLayer)
    pdb.python_layerfx_color_overlay(img, textLayer,
							  (color),   #color
									100,     #opacity
									 0,     #mode normal  
								   TRUE)    #merge
    textLayer = pdb.gimp_image_get_active_drawable(img) 
    
    t_width = pdb.gimp_drawable_width(textLayer)
    t_height = pdb.gimp_drawable_height(textLayer)
    
    #Duplicate the text
    textCopy = pdb.gimp_layer_new_from_drawable(textLayer, img)
    pdb.gimp_image_insert_layer(img, textCopy, None, 0)
    pdb.gimp_layer_set_mode(textCopy, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(textCopy, 100)
    pdb.gimp_image_select_item(img, 2, textCopy)
    pdb.gimp_selection_grow(img, 2)
    pdb.gimp_context_set_foreground(color)
    pdb.gimp_edit_bucket_fill(textCopy,  
                        BUCKET_FILL_FG,  #fill_mode
                     LAYER_MODE_NORMAL,  #paint_mode
                                   100,  #opacity
                                   255,  #threshold
                                 FALSE,  #sample_merged
                                     0,  #x
                                     0)  #y
    pdb.gimp_selection_none(img)
    
    #Text drop shadow
    color = "#000000"
    opacity = 75.0
    contour = 0
    noise = 0.0
    mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
    spread = 40.0
    size = 8
    offsetangle = 120.0
    offsetdist = 9.0
    knockout = TRUE
    merge = FALSE
    pdb.python_layerfx_drop_shadow(img, textLayer, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    
    #Text inner glow
    color = ("#6c5940")
    opacity = 75.0
    contour = 0
    noise = 0.0
    mode = 5        #LAYER_MODE_OVERLAY_LEGACY
    source = 0
    choke = 0.0
    size = 8
    merge = TRUE
    pdb.python_layerfx_inner_glow(img, textLayer, color, opacity, contour, noise, mode, source, choke, size, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    
    #Create the rust texture
    new_size = 300
    rust_Layer = pdb.gimp_layer_new(img, new_size, new_size, RGBA_IMAGE, "Rust texture", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, rust_Layer, None, 0)
    pdb.gimp_drawable_fill(rust_Layer, FILL_TRANSPARENT)
    pdb.plug_in_plasma(img, rust_Layer, random.randrange(1,999999999), 5)
    pdb.gimp_drawable_desaturate(rust_Layer,  DESATURATE_LIGHTNESS)
    #Shadow color balance
    pdb.gimp_drawable_color_balance(rust_Layer, 
                                             0,   #transfer_mode - shadows
                                          TRUE,   #preserve_lum
                                           100,   #cyan_red
                                           -15,   #magenta_green,
                                           -67)   #yellow_blue
	#Midtones color balance
    pdb.gimp_drawable_color_balance(rust_Layer, 
                                             1,   #transfer_mode - midtones
                                          TRUE,   #preserve_lum
                                           100,   #cyan_red
                                           -17,   #magenta_green,
                                           -55)   #yellow_blue                                 
	#Hightlights color balance
    pdb.gimp_drawable_color_balance(rust_Layer, 
                                             2,   #transfer_mode - hightlight
                                          TRUE,   #preserve_lum
                                           100,   #cyan_red
                                           -60,   #magenta_green,
                                          -100)   #yellow_blue      
    
    pdb.plug_in_colortoalpha(img, rust_Layer, ("#ffffff"))
    non_empty = pdb.gimp_edit_copy(rust_Layer)
    pdb.gimp_image_remove_layer(img, rust_Layer)
    
    #Text bevel and emboss
    style = 1                      #Inner bevel
    depth = 65
    direction = 0                  #Up
    size = 7
    soften = 1
    angle = 50.0
    altitude = 45.0
    glosscontour = 0                #Linear
    highlightcolor = ("#ffffff")
    highlightmode = 4                #LAYER_MODE_SCREEN_LEGACY
    highlightopacity = 80.0
    shadowcolor = ("#7b7b7b")
    shadowmode = 3                  #LAYER_MODE_MULTIPLY_LEGACY
    shadowopacity = 35.0
    surfacecontour = 0              #Linear
    use_texture = TRUE
    pattern = _("Clipboard Image")
    scale = 50.0
    tex_depth = 50.0
    invert = FALSE
    merge = TRUE
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
                                    style, depth, direction, size, soften, angle, altitude, glosscontour, highlightcolor, highlightmode, 
                                    highlightopacity, shadowcolor, shadowmode, 
                                    shadowopacity, surfacecontour, use_texture, pattern, scale, tex_depth, invert, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")

    #Text pattern overlay
    pattern =  _("Clipboard Image")
    opacity = 100.0
    mode = 5                       #LAYER_MODE_OVERLAY_LEGACY,
    scale = 50.0
    interpolation_type = INTERPOLATION_NONE
    merge = TRUE
    pdb.python_layerfx_pattern_overlay(img, textLayer, pattern, opacity, mode, scale, interpolation_type, merge)
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    
    #Text pattern overlay
    pattern =  _("Clipboard Image")
    opacity = 100.0
    mode = 5                       #LAYER_MODE_OVERLAY_LEGACY,
    scale = 30.0
    interpolation_type = INTERPOLATION_NONE
    merge = TRUE
    pdb.python_layerfx_pattern_overlay(img, textCopy, pattern, opacity, mode, scale, interpolation_type, merge)
    textCopy = pdb.gimp_image_get_layer_by_name(img, "Text copy")
    pdb.gimp_layer_set_mode(textCopy, LAYER_MODE_SUBTRACT)
    
    #Make it realistic
    pdb.gimp_drawable_hue_saturation(textCopy, ALL_HUES, 0, -50, 100, 0)
    textCopy = pdb.gimp_image_merge_down(img, textCopy, CLIP_TO_IMAGE)
    
    #Process
    pdb.gimp_displays_flush()

    #pdb.gimp_image_undo_group_end(img)	
	
register(
    "pm_rusty_text_effect_alt",
    "Creates a rusty effect",
    "Creates a rusty effect",
    "Pocholo",
    "Pocholo",
    "2022",
    "Rusty Effect",
    "",
    [
    (PF_IMAGE, "img", "Input image", None),    #current image
    (PF_DRAWABLE, "layer", "Input layer", None), #current layer
    (PF_COLOR, "color", "Color", (73, 55, 40)),
    ],
    [],
    pm_rusty_text_effect_alt, menu="<Image>/Filters/Alpha to Logo")

register(
    "pm_rusty_text_effect",
    "Creates a rusty text effect",
    "Creates a rusty text effect",
    "Pocholo",
    "Pocholo",
    "2022",
    "Rusty Text",
    "",
    [
    (PF_STRING, "customtext", "Text string", "RUST"),
    (PF_FONT, "font", "Font face", "Pieces of Eight"),
    (PF_COLOR, "textcolor", "Text color", (73, 55, 40)),
    ],
    [],
    pm_rusty_text_effect, menu="<Image>/Pocholo's-Plugins",
    domain=("gimp20", gimp.locale_directory))
main()	